VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MciCDPlayer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' MCI CD Player Class
'
' 30 September 2001
' 09 March 2002 - Discovered strange problems in Windows XP
'                 play, seek and stop are very slow.
'                 seek has a malfunction
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Settings

#Const PlayShareable = True
#Const NoOpenError = False



'------------------------------------------------------------------
' Private Variables

Private MCIPlayer As New MCIDevice

'------------------------------------------------------------------
' Public Enumerations

Public Enum CdTimeFormat
  CdMilliseconds = 1
  CdMinutesSecondsFrames = 2
  CdTracksMinutesSecondsFrames = 3
End Enum

Public Enum CdSoundSpeakers
  CdSpeakerLeft = 1
  CdSpeakerRight = 2
  CdSpeakerBoth = (CdSpeakerLeft Or CdSpeakerRight)
End Enum

Public Enum CdTrackType
  CdAudio = 0
  CdOther = 1
End Enum


'------------------------------------------------------------------
' Other Required


Private Sub Class_Initialize()
  #If NoOpenError Then
    Err.Clear
    On Error Resume Next
  #End If
  
  Call MCIPlayer.Attach("CDPlayer_" & ObjPtr(Me), 0)
  #If PlayShareable Then
    Call MCIPlayer.SendCommand("open cdaudio alias <ID> shareable")
  #Else
    Call MCIPlayer.SendCommand("open cdaudio alias <ID>")
  #End If
  
  #If NoOpenError Then
    Err.Clear
    On Error GoTo 0
  #End If
End Sub

Private Sub Class_Terminate()
  Dim State As String
  
  ' Read state (Access CD only if we really have to (avoids spin-up))
  State = MCIPlayer.SendCommand("status <ID> mode")
  If State <> "stopped" And State <> "open" Then
    Call MCIPlayer.SendCommand("stop <ID>")
  End If
  
  ' Close CD
  Call MCIPlayer.SendCommand("close <ID>")
End Sub

Public Property Get Wait() As Boolean
  Wait = MCIPlayer.Wait
End Property

Public Property Let Wait(NewValue As Boolean)
  MCIPlayer.Wait = NewValue
End Property

Public Property Let AbortKey(Optional Wait As Boolean, Optional hWndNotifyTo As Long, NewValue As Long)
  Call MCIPlayer.SendCommand("break <ID>" & IIf((NewValue = 0), (" off"), (" on " & NewValue)))
End Property

Public Property Get hWndNotifyTo() As Long
  hWndNotifyTo = MCIPlayer.hWndNotify
End Property

Public Property Let hWndNotifyTo(NewValue As Long)
  MCIPlayer.hWndNotify = NewValue
End Property

Public Property Let DoorOpen(NewValue As Boolean)
  Call MCIPlayer.SendCommand("set <ID> door " & IIf(NewValue, "open", "closed"))
End Property

Public Property Get DoorOpen() As Boolean
  DoorOpen = ("open" = MCIPlayer.SendCommand("status <ID> mode"))
End Property

Public Property Let TimeFormat(NewValue As CdTimeFormat)
  Dim TimeFormatStr As String
  Select Case NewValue
    Case CdMilliseconds:               TimeFormatStr = "milliseconds"
    Case CdMinutesSecondsFrames:       TimeFormatStr = "msf"
    Case CdTracksMinutesSecondsFrames: TimeFormatStr = "tmsf"
    Case Else:                         Error 5
  End Select
  Call MCIPlayer.SendCommand("set <ID> time format " & TimeFormatStr)
End Property

Public Property Get TimeFormat() As CdTimeFormat
  Dim TimeFormatStr As String
  TimeFormatStr = MCIPlayer.SendCommand("status <ID> time format")
  Select Case TimeFormatStr
    Case "milliseconds": TimeFormat = CdMilliseconds
    Case "msf":          TimeFormat = CdMinutesSecondsFrames
    Case "tmsf":         TimeFormat = CdTracksMinutesSecondsFrames
    Case Else:           Err.Raise 12345, "CdPlayer", "TimeFormat failed: (Format=" & TimeFormatStr & ") !"
  End Select
End Property

Public Sub PlayCD(Optional FromPos As Long, Optional ToPos As Long)
  Call MCIPlayer.SendCommand("play <ID>" & IIf(FromPos, " from " & FromPos, "") & IIf(ToPos, " to " & ToPos, ""))
End Sub

Public Sub StopCD()
  Call MCIPlayer.SendCommand("stop <ID>")
  Me.Position = Me.TrackStart(Me.Track) ' Force to start of track
End Sub

Public Property Get Track() As Long
  Track = CLng(MCIPlayer.SendCommand("status <ID> current Track"))
End Property

Public Property Let Track(NewValue As Long)
  Me.Position = Me.TrackStart(NewValue)
End Property

Public Property Get TrackType(Track As Long) As CdTrackType
  Dim TypeStr As String
  TypeStr = MCIPlayer.SendCommand("status <ID> type track " & Track)
  If TypeStr <> "audio" Then TrackType = CdOther ' Avoid language trouble
End Property

Public Property Get Tracks() As Long
  Tracks = CLng(MCIPlayer.SendCommand("status <ID> number of tracks"))
End Property

Public Property Let Position(NewValue As Variant)
  If ("playing" = MCIPlayer.SendCommand("status <ID> mode")) Then
    Call MCIPlayer.SendCommand("play <ID> from " & NewValue)
  Else
    Call MCIPlayer.SendCommand("seek <ID> to " & NewValue)
  End If
End Property

Public Property Get Position() As Variant
  Dim Pos As String
  Pos = MCIPlayer.SendCommand("status <ID> position")
  If CStr(Val(Pos)) = Pos Then
    Position = Val(Pos)
  Else
    Position = Pos
  End If
End Property

Public Property Get TrackStart(Track As Long) As Variant
  Dim Pos As String
  Pos = MCIPlayer.SendCommand("status <ID> position track " & Track)
  If CStr(Val(Pos)) = Pos Then
    TrackStart = Val(Pos)
  Else
    TrackStart = Pos
  End If
End Property

Public Property Get TrackLength(Track As Long) As String
  Dim Pos As String
  Pos = MCIPlayer.SendCommand("status <ID> length track " & Track)
  If CStr(Val(Pos)) = Pos Then
    TrackLength = Val(Pos)
  Else
    TrackLength = Pos
  End If
End Property

Public Property Get Start() As Variant
  Dim Pos As String
  Pos = MCIPlayer.SendCommand("status <ID> start position")
  If CStr(Val(Pos)) = Pos Then
    Start = Val(Pos)
  Else
    Start = Pos
  End If
End Property

Public Property Get Length() As Variant
  Dim LastTrack As Long
  Dim CurFormat As CdTimeFormat
  
  CurFormat = Me.TimeFormat
  If CurFormat = CdMilliseconds Then
    Length = CLng(MCIPlayer.SendCommand("status <ID> length"))
  Else
    If CurFormat = CdTracksMinutesSecondsFrames Then
      LastTrack = Me.Tracks
      ' Need to convert to tmsf
      Length = Format(LastTrack, "00") & ":" & Me.TrackLength(LastTrack)
    ElseIf CurFormat = CdMinutesSecondsFrames Then
      Length = MCIPlayer.SendCommand("status <ID> length")
    End If
  End If
End Property

Public Property Get Ready() As Boolean
  Ready = CBool(MCIPlayer.SendCommand("status <ID> ready"))
End Property

Public Property Get Present() As Boolean
  Present = CBool(MCIPlayer.SendCommand("status <ID> media present"))
End Property

Public Property Get State() As String 'not ready|paused|playing|stopped|[open|parked|recording|seeking]
  State = MCIPlayer.SendCommand("status <ID> mode")
End Property

Public Sub GotoStart()
  Call MCIPlayer.SendCommand("seek <ID> to start")
End Sub

Public Sub GotoEnd()
  Call MCIPlayer.SendCommand("seek <ID> to end")
End Sub

Public Property Let Paused(NewValue As Boolean)
  ' resume command not supported by each device
  If NewValue Then
    Call MCIPlayer.SendCommand("pause <ID>")
  Else
    Call MCIPlayer.SendCommand("play <ID>")
  End If
End Property

Public Property Get Paused() As Boolean
  Paused = ("stopped" = MCIPlayer.SendCommand("status <ID> mode"))
End Property

Public Property Get Stopped() As Boolean
  Stopped = ("stopped" = MCIPlayer.SendCommand("status <ID> mode"))
End Property

Public Property Get Playing() As Boolean
  Playing = ("playing" = MCIPlayer.SendCommand("status <ID> mode"))
End Property

Public Property Let SoundOn(Optional Speaker As CdSoundSpeakers = CdSpeakerBoth, NewValue As Boolean)
  Dim SpeakerID As String
  Select Case Speaker
    Case CdSpeakerBoth:  SpeakerID = " all "
    Case CdSpeakerLeft:  SpeakerID = " left "
    Case CdSpeakerRight: SpeakerID = " right "
  End Select
  Call MCIPlayer.SendCommand("set <ID> audio" & SpeakerID & IIf(NewValue, "on", "off"))
End Property

Public Property Get ID() As Variant
  ID = MCIPlayer.SendCommand("info <ID> identity")
End Property

Public Property Get UPC() As Variant
  UPC = MCIPlayer.SendCommand("info <ID> upc")
End Property

Public Property Get ProductInfo() As String
  ProductInfo = MCIPlayer.SendCommand("info <ID> product")
End Property
